/*
 * Copyright (C) 2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/grpc_client/export_scan_tasks.hh"
#include "src/grpc_client/make_stub.hh"
#include "src/grpc_client/proto_wrap.hh"
#include "src/exception.hh"

#include <grpc/grpc.h>
#include <grpcpp/client_context.h>

#include <liblog/liblog.hh>

namespace Fred {
namespace Akm {
namespace GrpcClient {

void export_scan_tasks(const SupervisorConnectionString& _supervisor_connection_string, const ScanTasks& _scan_tasks)
{
    namespace Api = CdnskeyProcessor::Api::Supervisor;

    auto stub = make_stub(_supervisor_connection_string);
    grpc::ClientContext context;
    ::google::protobuf::Empty response;
    auto writer = stub->import_tasks(&context, &response);//client exports, server imports !?

    LIBLOG_TRACE("{} domains to export", _scan_tasks.size());
    int write_count = 0;
    for (auto chunk_begin = begin(_scan_tasks); chunk_begin != end(_scan_tasks);)
    {
        static constexpr auto max_chunk_size = 100;
        auto chunk_end = chunk_begin;
        for (auto chunk_size = 0; (chunk_size < max_chunk_size) && (chunk_end != end(_scan_tasks)); ++chunk_end, ++chunk_size) { }
        const auto scan_tasks_api = proto_wrap(chunk_begin, chunk_end);
        const auto stream_ok = writer->Write(scan_tasks_api);
        if (!stream_ok)
        {
            LIBLOG_INFO("Write() failed: bad stream");
            throw GrpcException{"bad stream"};
        }
        ++write_count;
        chunk_begin = chunk_end;
    }
    LIBLOG_TRACE("{} successful writes", write_count);
    const auto success = writer->WritesDone();
    if (!success)
    {
        LIBLOG_INFO("WritesDone() failed");
        throw GrpcException{"WritesDone() failed"};
    }
    const auto status = writer->Finish();
    if (!status.ok())
    {
        LIBLOG_INFO("Finish() failed: {}", status.error_message());
        throw GrpcException{"status not OK: " + status.error_message()};
    }
    LIBLOG_TRACE("done");
}

} // namespace Fred::Akm::GrpcClient
} // namespace Fred::Akm
} // namespace Fred
