/*
 * Copyright (C) 2021-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef LOG_ENTRY_HH_B0667E1A9AA096499A2A841E2AE22F31//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define LOG_ENTRY_HH_B0667E1A9AA096499A2A841E2AE22F31

#include "src/domain.hh"
#include "src/keyset.hh"
#include "src/keyset_reference.hh"
#include "src/logger/impl/log_entry_type.hh"

#include <libfred/registrable_object/keyset/keyset_dns_key.hh>
#include <libgrill/connection.hh>
#include <libgrill/logger/log_entry_id.hh>

namespace Fred {
namespace Akm {
namespace Logger {

using LogEntryNumericalId = LibGrill::Logger::LogEntryId::NumericalId;

template <LogEntryType>
class LogEntry
{
public:
    ~LogEntry();
    template<typename... Args>
    void close_on_success(const KeysetReference&, const Args&...);
    void close_on_failure();
    LogEntryNumericalId get_id() const;
private:
    LogEntry(
            LibGrill::Connection<LibGrill::Service::Logger>&,
            const LibGrill::Logger::LogEntryId&,
            const DomainFqdn&);
    bool skip_any_close_call_;
    LibGrill::Connection<LibGrill::Service::Logger>& connection_;
    LibGrill::Logger::LogEntryId log_entry_id_;
    DomainFqdn domain_fqdn_;
    friend class LogEntryFactory;
};

template<>
template<>
void LogEntry<LogEntryType::akm_turn_off>::close_on_success(const KeysetReference&);

template<>
template<>
void LogEntry<LogEntryType::akm_turn_on>::close_on_success(const KeysetReference&, const Keyset&);

template<>
template<>
void LogEntry<LogEntryType::akm_rollover>::close_on_success(
        const KeysetReference&,
        const std::vector<LibFred::DnsKey>& _old,
        const Keyset& _new);

} // namespace Fred::Akm::Logger
} // namespace Fred::Akm
} // namespace Fred

#endif//LOG_ENTRY_HH_B0667E1A9AA096499A2A841E2AE22F31
