/*
 * Copyright (C) 2021-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/grpc_client/get_acceptable_keysets.hh"
#include "src/database/process_acceptable_keysets.hh"
#include "src/logger/log_entry_factory.hh"
#include "src/exception.hh"
#include "src/update.hh"

#include <liblog/liblog.hh>

namespace Fred {
namespace Akm {

void update_acceptable_keysets(
        const KeysetMetadata& _keyset_metadata,
        const SupervisorConnectionString& _supervisor_connection_string,
        const Logger::ConnectionString& _logger_connection_string,
        DryRun dry_run)
{
    try
    {
        const auto acceptable_keysets = GrpcClient::get_acceptable_keysets(_supervisor_connection_string);
        Logger::Connection logger_connection{_logger_connection_string};
        Logger::LogEntryFactory log_entry_factory(logger_connection);
        Database::process_acceptable_keysets(_keyset_metadata, acceptable_keysets, log_entry_factory, dry_run);
    }
    catch (const ProtoException& e)
    {
        LIBLOG_ERROR("can't unwrap AcceptableDnskey message: {}", e.what());
        throw;
    }
    catch (const GrpcException& e)
    {
        LIBLOG_ERROR("gRPC call exception: {}", e.what());
        throw;
    }
    catch (...)
    {
        throw;
    }
}

} // namespace Fred::Akm
} // namespace Fred
